@@ -4,7 +4,7 @@
 modname = 'container'
 distname = 'cubicweb-container'
 
-numversion = (2, 2, 0)
+numversion = (2, 3, 0)
 version = '.'.join(str(num) for num in numversion)
 
 license = 'LGPL'
@@ -13,7 +13,8 @@ author_email = 'contact@logilab.fr'
 description = 'provides "generic container" services'
 web = 'http://www.cubicweb.org/project/%s' % distname
 
-__depends__ =  {'cubicweb': '>= 3.15.10'}
+__depends__ =  {'cubicweb': '>= 3.15.10',
+                'yams': '>= 0.38.1'}
 __recommends__ = {}
 
 
