@@ -1,4 +1,4 @@
-# copyright 2011-2012 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+# copyright 2011-2014 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 # contact http://www.logilab.fr -- mailto:contact@logilab.fr
 #
 # This program is free software: you can redistribute it and/or modify it under
@@ -15,14 +15,18 @@
 # with this program. If not, see <http://www.gnu.org/licenses/>.
 
 """cubicweb-container entity's classes"""
+from datetime import datetime
 from collections import defaultdict
+from itertools import izip, chain
 from warnings import warn
 
 from logilab.common.decorators import cached, cachedproperty
 
 from rql import parse
 
+from cubicweb import Binary
 from cubicweb.server.ssplanner import READ_ONLY_RTYPES
+from cubicweb.server.utils import eschema_eid
 
 from cubicweb.schema import VIRTUAL_RTYPES
 from cubicweb.entities import AnyEntity
@@ -33,6 +37,7 @@ from cubes.container.utils import (yet_unset,
                                    container_rtypes_etypes,
                                    parent_rschemas,
                                    needs_container_parent,
+                                   _insertmany,
                                    _add_rqlst_restriction,
                                    _iter_mainvar_relations)
 
@@ -82,6 +87,8 @@ class ContainerProtocol(EntityAdapter):
         parent = self.parent
         if parent:
             container = parent.cw_adapt_to('Container').related_container
+            if container is None:
+                return
             if self.entity.e_schema not in container.container_skipetypes:
                 return container
 
@@ -172,8 +179,9 @@ class ContainerClone(EntityAdapter):
         # let's flush all collected relations
         self.info('linking (%d relations)', len(relations))
         for rtype, eids in relations.iteritems():
-            self.info('%s linking %s' %
-                      ('internal' if rtype in internal_rtypes else 'external', rtype))
+            self.info('%s linking %s (%s elements)' %
+                      ('internal' if rtype in internal_rtypes else 'external',
+                       rtype, len(eids)))
             subj_obj = []
             for subj, obj in eids:
                 subj = orig_to_clone[subj]
@@ -196,7 +204,7 @@ class ContainerClone(EntityAdapter):
             # the orig-clone mapping and relations will be augmented
             # by the delegated clone
             cloner._clone(orig_to_clone, relations, toplevel=False)
-
+            cloner._container_relink(orig_to_clone)
 
     @cachedproperty
     def clone_rtype(self):
@@ -238,10 +246,9 @@ class ContainerClone(EntityAdapter):
 
     @cachedproperty
     def _no_copy_meta(self):
-        # handled unconditionnally by the native source, hence we must not touch them
-        # ALSO include cwuri, which is mandatory in cw but maybe should not
+        # include cwuri, which is mandatory in cw but maybe should not
         # (see cw#3267139)
-        return set(('cw_source', 'cwuri')) | READ_ONLY_RTYPES | VIRTUAL_RTYPES
+        return set(('cwuri',)) | READ_ONLY_RTYPES | VIRTUAL_RTYPES
 
     def _etype_fetch_rqlst(self, etype):
         """ returns an rqlst ready to be executed, plus a sequence of
@@ -254,7 +261,7 @@ class ContainerClone(EntityAdapter):
         # a dict from rtype to rql variable
         already_used_rtypes = dict(_iter_mainvar_relations(rqlst))
         # running without metadata hooks: we must handle some rtypes here
-        # we also will loose: is_instance_of, created_by, owned_by
+        # we also will loose: is_instance_of, cwuri, has_text, eid, is, identity
         no_copy_meta = self._no_copy_meta
         # keep an ordered-list of selected rtypes
         fetched_rtypes = []
@@ -297,6 +304,10 @@ class ContainerClone(EntityAdapter):
                     or rtype in self.rtypes_to_skip):
                 yield rtype
 
+    @cachedproperty
+    def _ordered_etypes(self):
+        return list(self.clonable_etypes())
+
     def clonable_etypes(self):
         for etype in ordered_container_etypes(self._cw.vreg.schema,
                                               self.entity.__regid__,
@@ -337,31 +348,157 @@ class ContainerClone(EntityAdapter):
             self.handle_special_relations((rtype, orig_to_clone[orig], linked)
                                           for rtype, orig, linked in deferred_relations)
 
+    def _fast_reserve_eids(self, qty):
+        """ not fast enough (yet) """
+        source = self._cw.repo.sources_by_uri['system']
+        if qty == 1:
+            return (source.create_eid(self._cw),)
+        return source.create_eid(self._cw, count=qty)
+
+    def preprocess_attributes(self, etype, oldeid, attributes):
+        pass
+
+    def _fast_create_entities(self, etype, entities, orig_to_clone):
+        eschema = self._cw.vreg.schema[etype]
+        etypeid = eschema_eid(self._cw, eschema)
+        ancestorseid = [etypeid] + [eschema_eid(self._cw, aschema)
+                                    for aschema in eschema.ancestors()]
+        metadata = []
+        isrelation = []
+        isinstanceof = []
+        now = datetime.utcnow()
+        for attributes in entities:
+            neweid = attributes['eid']
+            metadata.append({'type': etype, 'eid': neweid,
+                             'source': 'system', 'asource': 'system',
+                             'mtime': now})
+            isrelation.append({'eid_from': neweid, 'eid_to': etypeid})
+            for ancestor in ancestorseid:
+                isinstanceof.append({'eid_from': neweid, 'eid_to': ancestor})
+
+        # insert entities
+        _insertmany(self._cw, etype, entities, prefix='cw_')
+        # insert metadata
+        _insertmany(self._cw, 'entities', metadata)
+        _insertmany(self._cw, 'is_relation', isrelation)
+        _insertmany(self._cw, 'is_instance_of_relation', isinstanceof)
+
+    def _crosses_border(self, etype, rtype):
+        """ Tells whether the (etype, rtype, *) relation
+        has ALL its targets outside of the container """
+        if rtype == 'container_parent':
+            # it is technically possible that it crosses the border
+            # but a container_parent, by design, is always an inner
+            # container relation
+            return False
+        schema = self._cw.vreg.schema
+        etypes = set(self._ordered_etypes)
+        etypes.add(self.entity.cw_etype)
+        return all(target not in etypes
+                   for target in schema[rtype].targets(etype))
+
+    def _already_cloned(self, etype, rtype):
+        """ Tells whether all targets in the (etype, rtype, TARGET)
+        relation can have been already cloned """
+        # a shortcut for the container relation
+        if rtype == self.entity.container_rtype:
+            return True
+        schema = self._cw.vreg.schema
+        ordered_etypes = self._ordered_etypes
+        try:
+            return all(ordered_etypes.index(etype) > ordered_etypes.index(target)
+                       for target in schema[rtype].targets(etype))
+        except ValueError:
+            # some target is not clonable
+            return False
+
     def _etype_create_clones(self, etype, orig_to_clone, candidates_rset,
                              relations, deferred_relations,
                              fetched_rtypes, inlined_rtypes):
-        create = self._cw.create_entity
-        for row in candidates_rset.rows:
-            candidate_eid = row[0]
-            attributes = {}
-            for rtype, val in zip(fetched_rtypes, row[1:]):
-                if val is None:
+        entities = []
+        neweids = self._fast_reserve_eids(len(candidates_rset))
+
+        # detect inlined rtypes that may be already cloned
+        inlined_rtypes_already_cloned = set()
+        # detect inlined rtypes that have at least one
+        # out-of-container target
+        inlined_rtypes_crossing_border = set()
+
+        for rtype in fetched_rtypes:
+            if rtype in inlined_rtypes:
+                if self._crosses_border(etype, rtype):
+                    inlined_rtypes_crossing_border.add(rtype)
+                elif self._already_cloned(etype, rtype):
+                    inlined_rtypes_already_cloned.add(rtype)
+
+        # Unfortunately, the above classification still can miss
+        # opportunities, because the static analysis lacks relevant
+        # information.
+        # Hence we peek the first rset row to determine if an inlined
+        # relation has been cloned, and if by chance it is valued
+        # and appears to have a clone, we just avoided to send an inlined
+        # relation to .add_relations (which performs horribly).
+        iterrows = iter(candidates_rset.rows)
+        firstrow = iterrows.next()
+        for rtype, val in zip(fetched_rtypes, firstrow[1:]):
+            if rtype in inlined_rtypes:
+                if rtype in inlined_rtypes_crossing_border:
                     continue
+                if val in orig_to_clone:
+                    if rtype not in inlined_rtypes_crossing_border:
+                        inlined_rtypes_already_cloned.add(rtype)
+
+        # Let's gather some real-life info
+        self.info('Optimized inlined rtypes for %s', etype)
+        self.info('crossing border (no need to wait for a clone): %s',
+                  sorted(inlined_rtypes_crossing_border))
+        self.info('already cloned: %s',
+                  sorted(inlined_rtypes_already_cloned))
+        self.info('unoptimisable: %s', sorted(inlined_rtypes -
+                                              (inlined_rtypes_already_cloned |
+                                               inlined_rtypes_crossing_border)))
+
+        for row, neweid in izip(chain([firstrow], iterrows), neweids):
+            oldeid = row[0]
+            attributes = {'eid': neweid, 'cwuri': u''}
+            for rtype, val in zip(fetched_rtypes, row[1:]):
+
                 if rtype in inlined_rtypes:
+                    # We handle these carefully to ensure the following invariant:
+                    # either an rtype is handled in attributes or in relation
+                    # but this is not a value-dependant decision
+                    # (because insertmany is a bit rigid).
+                    # Hence, even if val is None, we take it
                     if rtype in self._specially_handled_rtypes:
-                        deferred_relations.append((rtype, candidate_eid, val))
-                    elif val in orig_to_clone:
-                        attributes[rtype] = orig_to_clone[val]
-                    else:
-                        relations[rtype].append((candidate_eid, val))
-                else: # standard attribute
-                    attributes[rtype] = val
-            # work around cwuri obnoxiousness
-            if 'cwuri' not in attributes:
-                attributes['cwuri'] = u''
-            clone = create(etype, **attributes)
-            clone.cw_clear_all_caches()
-            orig_to_clone[candidate_eid] = clone.eid
+                        deferred_relations.append((rtype, oldeid, val))
+                        continue
+
+                    if rtype in inlined_rtypes_already_cloned:
+                        # there can be Nones here
+                        if val is not None:
+                            assert val in orig_to_clone
+                        attributes[rtype] = orig_to_clone.get(val)
+                        continue
+
+                    if rtype in inlined_rtypes_crossing_border:
+                        # feed it right away
+                        attributes[rtype] = val
+                        continue
+
+                    # deferred to relations (or nothing if None)
+                    if val is not None:
+                        relations[rtype].append((oldeid, val))
+                    continue
+
+                # standard attribute
+                if isinstance(val, Binary):
+                    val = buffer(val.getvalue())
+                attributes[rtype] = val
+
+            self.preprocess_attributes(etype, oldeid, attributes)
+            orig_to_clone[oldeid] = neweid
+            entities.append(attributes)
+        self._fast_create_entities(etype, entities, orig_to_clone)
 
     def _etype_relink_clones(self, etype, queryargs, relations, deferred_relations):
         etype_rql = self._complete_rql(etype)
@@ -382,25 +519,29 @@ class ContainerClone(EntityAdapter):
     def _container_relink(self, orig_to_clone):
         """ handle subject relations of the container - this is
         handled specially because attributes have already been set
+
+        we partition subject relations in two sets:
+        * those that are already set (typically .owned_by, .created_by, ...)
+        * those that must yet be cloned
         """
         deferred_relations = []
         relations = defaultdict(list)
         queryargs = self._queryargs()
-        ceschema = self.entity.e_schema
-        etype = ceschema.type
+        clone = self.entity
+        etype = clone.e_schema.type
         etype_rql = 'Any X WHERE X is %s, X eid %s' % (
                 etype, self.orig_container_eid)
         skiprtypes = set(self.rtypes_to_skip)
         if self.clone_rtype:
             skiprtypes.add(self.clone_rtype)
-        for rschema in ceschema.subject_relations():
-            if rschema.final:
-                continue
-            rtype = rschema.type
-            if rtype in skiprtypes:
-                continue
-            if rtype not in VIRTUAL_RTYPES:
-                continue
+        clone_subject_relations = set(rschema.type
+                                      for rschema in clone.e_schema.subject_relations()
+                                      if not rschema.final)
+        for rtype in self.clonable_rtypes(etype):
+            if rtype in clone_subject_relations:
+                if self._cw.execute('Any Y LIMIT 1 WHERE X eid %%(clone)s, X %(rtype)s Y'
+                                    % {'rtype': rtype}, {'clone': clone.eid}):
+                    continue
             etype_rqlst = parse(etype_rql).children[0]
             _add_rqlst_restriction(etype_rqlst, rtype)
             linked_rset = self._cw.execute(etype_rqlst, queryargs)
