@@ -1,8 +1,24 @@
+# copyright 2011-2014 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+# contact http://www.logilab.fr -- mailto:contact@logilab.fr
+#
+# This program is free software: you can redistribute it and/or modify it under
+# the terms of the GNU Lesser General Public License as published by the Free
+# Software Foundation, either version 2.1 of the License, or (at your option)
+# any later version.
+#
+# This program is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
+# details.
+#
+# You should have received a copy of the GNU Lesser General Public License along
+# with this program. If not, see <http://www.gnu.org/licenses/>.
+
 from collections import deque, defaultdict
 from warnings import warn
 import logging
 
-from logilab.common.decorators import cached
+from logilab.common.decorators import cached, monkeypatch
 from logilab.common.deprecation import deprecated
 
 from yams.buildobjs import RelationType, RelationDefinition
@@ -13,6 +29,8 @@ from rql.nodes import Comparison, VariableRef, make_relation
 from cubicweb import neg_role, schema as cw_schema
 from cubicweb.appobject import Predicate
 
+from cubicweb.server.sources.native import NativeSQLSource
+
 logger = logging.getLogger()
 
 class yet_unset(Predicate):
@@ -25,9 +43,9 @@ def composite_role(eschema, rschema):
     """ testing compositeness is a bit awkward with the standard
     yams API (due to potentially multirole relation definitions) """
     try:
-        return eschema.rdef(rschema, 'subject').composite
+        return eschema.rdef(rschema, 'subject', takefirst=True).composite
     except KeyError:
-        return eschema.rdef(rschema, 'object').composite
+        return eschema.rdef(rschema, 'object', takefirst=True).composite
 
 @cached
 def _composite_rschemas(eschema):
@@ -35,7 +53,7 @@ def _composite_rschemas(eschema):
     for rschema, _types, role in eschema.relation_definitions():
         if rschema.meta or rschema.final:
             continue
-        crole = eschema.rdef(rschema, role).composite
+        crole = eschema.rdef(rschema, role, takefirst=True).composite
         if crole:
             output.append( (rschema, role, crole) )
     return output
@@ -68,6 +86,16 @@ def needs_container_parent(eschema):
 
 def define_container(schema, cetype, crtype, rtype_permissions=None,
                      skiprtypes=(), skipetypes=(), subcontainers=()):
+    """Handle container definition in schema
+
+    * insert the container relation type `crtype` in schema (possibly with
+      `rtype_permissions`) if not already present.
+
+    * insert all relation definitions `crtype` between the container
+      entity type and entities belonging to the container, the latter
+      being defined by construction of the container structure (see
+      `container_static_structure`).
+    """
     _rtypes, etypes = container_static_structure(schema, cetype, crtype,
                                                  skiprtypes=skiprtypes,
                                                  skipetypes=skipetypes,
@@ -114,8 +142,12 @@ def define_container_parent_rdefs(schema, etype,
 
 def container_static_structure(schema, cetype, crtype, skiprtypes=(), skipetypes=(),
                                subcontainers=()):
-    """ return etypes and composite rtypes (the rtypes
-    that _define_ the structure of the Container graph)
+    """Return the sets of entity types and relation types that define the
+    structure of the container.
+
+    The skeleton (or structure) of the container is determined by following
+    composite relations, possibly skipping specified entity types and/or
+    relation types.
     """
     skiprtypes = set(skiprtypes).union((crtype, 'container_etype', 'container_parent'))
     skipetypes = set(skipetypes)
@@ -207,7 +239,11 @@ def set_container_relation_rtypes_hook(schema, cetype, crtype, skiprtypes=(), sk
 
 def container_rtypes_etypes(schema, cetype, crtype, skiprtypes=(), skipetypes=(),
                             subcontainers=()):
-    """ returns set of rtypes, set of etypes of what is in a Container """
+    """Return all entity types and relation types that are part of the container.
+
+    It extends ``container_static_structure`` with non structural relation types
+    between entity types belonging to the defining structure of the container.
+    """
     skiprtypes = set(skiprtypes).union((crtype,'container_etype', 'container_parent'))
     rtypes, etypes = container_static_structure(schema, cetype, crtype,
                                                 skiprtypes, skipetypes, subcontainers)
@@ -328,12 +364,10 @@ def container_etype_orders(schema, cetype, crtype, skiprtypes=(), skipetypes=(),
                                            skiprtypes))
                      for etype in etypes)
     maplen = len(etype_map)
-    def _append_order():
+    while etype_map:
         neworder = linearize(etype_map, etypes)
         if neworder:
             orders.append(neworder)
-    while etype_map:
-        _append_order()
         if maplen == len(etype_map):
             break
         maplen = len(etype_map)
@@ -381,6 +415,80 @@ def _iter_mainvar_relations(rqlst):
                 yield rel.r_type, rel.children[1].children[0]
 
 
+def _insertmany(session, table, attributes, prefix=''):
+    """ Low-level INSERT many entities of the same etype
+    at once
+    """
+    # the low-level python-dbapi cursor
+    cursor = session.cnxset['system']
+    columns = sorted(attributes[0])
+    cursor.executemany('INSERT INTO %s (%s) VALUES (%s)' % (
+        prefix + table,                                    # table name
+        ','.join(prefix + name for name in columns),       # column names
+        ','.join('%%(%s)s' %  name for name in columns)),  # dbapi placeholders
+                       attributes)
+
+# clone: fast eid range creation
+@monkeypatch(NativeSQLSource)
+def _create_eid_sqlite(self, session, count=1, eids=None):
+    with self._eid_cnx_lock:
+        eids = []
+        for _x in xrange(count):
+            for sql in self.dbhelper.sqls_increment_sequence('entities_id_seq'):
+                cursor = self.doexec(session, sql)
+            eids.append(cursor.fetchone()[0])
+        if count > 1:
+            return eids
+        return eids[0]
+
+@monkeypatch(NativeSQLSource)
+def create_eid(self, session, count=1):
+    with self._eid_cnx_lock:
+        return self._create_eid(count)
+
+@monkeypatch(NativeSQLSource)
+def _create_eid(self, count, eids=None):
+    # internal function doing the eid creation without locking.
+    # needed for the recursive handling of disconnections (otherwise we
+    # deadlock on self._eid_cnx_lock
+    if self._eid_creation_cnx is None:
+        self._eid_creation_cnx = self.get_connection()
+    cnx = self._eid_creation_cnx
+    try:
+        eids = eids or []
+        cursor = cnx.cursor()
+        for _x in xrange(count):
+            for sql in self.dbhelper.sqls_increment_sequence('entities_id_seq'):
+                cursor.execute(sql)
+            eids.append(cursor.fetchone()[0])
+    except (self.OperationalError, self.InterfaceError):
+        # FIXME: better detection of deconnection pb
+        self.warning("trying to reconnect create eid connection")
+        self._eid_creation_cnx = None
+        return self._create_eid(count, eids)
+    except self.DbapiError as exc:
+        # We get this one with pyodbc and SQL Server when connection was reset
+        if exc.args[0] == '08S01':
+            self.warning("trying to reconnect create eid connection")
+            self._eid_creation_cnx = None
+            return self._create_eid(count, eids)
+        else:
+            raise
+    except Exception:
+        cnx.rollback()
+        self._eid_creation_cnx = None
+        self.exception('create eid failed in an unforeseen way on SQL statement %s', sql)
+        raise
+    else:
+        cnx.commit()
+        # one eid vs many
+        # we must take a list because the postgres sequence does not
+        # ensure a contiguous sequence
+        if count > 1:
+            return eids
+        return eids[0]
+
+
 # migration helper
 
 def synchronize_container_parent_rdefs(schema,
